.\" 
.\" Mach Operating System
.\" Copyright (c) 1993,1991,1990 Carnegie Mellon University
.\" All Rights Reserved.
.\" 
.\" Permission to use, copy, modify and distribute this software and its
.\" documentation is hereby granted, provided that both the copyright
.\" notice and this permission notice appear in all copies of the
.\" software, derivative works or modified versions, and any portions
.\" thereof, and that both notices appear in supporting documentation.
.\" 
.\" CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
.\" CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
.\" ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
.\" 
.\" Carnegie Mellon requests users of this software to return to
.\" 
.\"  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
.\"  School of Computer Science
.\"  Carnegie Mellon University
.\"  Pittsburgh PA 15213-3890
.\" 
.\" any improvements or extensions that they make and grant Carnegie Mellon
.\" the rights to redistribute these changes.
.\" 
.\" 
.\" HISTORY
.\" $Log:	processor_set_policy_limit.man,v $
.\" Revision 2.2  93/12/07  13:56:59  dbg
.\" 		Modified for processor_set_policy_limit.
.\" 	[93/09/22            dbg]
.\" 
.\" Revision 2.5  93/03/18  15:14:28  mrt
.\" 	corrected types
.\" 	[93/03/11  12:54:05  lli]
.\" 
.\" Revision 2.4  91/05/14  17:11:46  mrt
.\" 	Correcting copyright
.\" 
.\" Revision 2.3  91/02/14  14:14:07  mrt
.\" 	Changed to new Mach copyright
.\" 	[91/02/12  18:14:56  mrt]
.\" 
.\" Revision 2.2  90/08/07  18:42:50  rpd
.\" 	Created.
.\" 
.TH processor_set_policy_limit 2 9/22/93
.CM 4
.SH NAME
.nf
processor_set_policy_limit  \-  set maximum values for a policy on a
processor set
.SH SYNOPSIS
.nf
.ft B
#include <mach.h>

.nf
.ft B
kern_return_t
processor_set_policy_limit(
	mach_port_t		processor_set,
	int			policy,
	policy_param_t		limit,
	natural_t		limit_count,
	boolean_t		change_threads);


.fi
.ft P
.SH ARGUMENTS
.TP 15
.B
processor_set
Processor set to set maximum priority for.
.TP 15
.B
policy
The policy whose limit values are to be changed.
.TP 15
.B
limit
The new limit values for the policy.
.TP 15
.B
limit_count
Number of items (as natural_t) in
.B
limit
.TP 15
.B
change_threads
Change limit value of existing threads running
.B
policy
 if TRUE.

.SH DESCRIPTION
The policy limit values for a particular policy on a processor set are
used only for newly created threads running that policy
(thread's policy limit is set to processor set's) and the assignment
of threads to the set (thread's policy limit is reduced if it exceeds
the set's policy limit for the thread's policy; thread's policy
parameters are similarly reduced).
.B processor_set_policy_limit
changes this limit.  It also sets the limit values of
all threads assigned to the processor set 
to this new value if 
.B change_threads
is TRUE.  If this policy limit is less than the policy parameters of any
of these threads, their policy parameters will also be set to this new value.

.SH DIAGNOSTICS
.TP 25
KERN_SUCCESS
The call succeeded.
.TP 25
KERN_INVALID_ARGUMENT
.B task
is not a task, or
.B policy
is not currently enabled on the processor set, or
.B limit
is not a valid value for the policy.

.SH BUGS
Availability limited.

.SH SEE ALSO
.B thread_set_policy, thread_set_policy_param,
task_set_default_policy, thread_assign




