.\" 
.\" Mach Operating System
.\" Copyright (c) 1993,1991,1990 Carnegie Mellon University
.\" All Rights Reserved.
.\" 
.\" Permission to use, copy, modify and distribute this software and its
.\" documentation is hereby granted, provided that both the copyright
.\" notice and this permission notice appear in all copies of the
.\" software, derivative works or modified versions, and any portions
.\" thereof, and that both notices appear in supporting documentation.
.\" 
.\" CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
.\" CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
.\" ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
.\" 
.\" Carnegie Mellon requests users of this software to return to
.\" 
.\"  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
.\"  School of Computer Science
.\"  Carnegie Mellon University
.\"  Pittsburgh PA 15213-3890
.\" 
.\" any improvements or extensions that they make and grant Carnegie Mellon
.\" the rights to redistribute these changes.
.\" 
.\" 
.\" HISTORY
.\" $Log:	task_set_default_policy.man,v $
.\" Revision 2.2  93/12/07  13:57:54  dbg
.\" 	Created.
.\" 
.\" 
.TH task_set_default_policy 2 9/22/93
.CM 4
.SH NAME
.nf
task_set_default_policy  \-  set scheduling policy for a task
.SH SYNOPSIS
.nf
.ft B
#include <mach.h>

.nf
.ft B
kern_return_t task_set_default_policy(
	mach_port_t	task,
	processor_set_t	pset,
	int		policy,
	policy_param_t	param,
	natural_t	param_count,
	boolean_t	change_threads);



.fi
.ft P
.SH ARGUMENTS
.TP 15
.B
task
Task to set policy for.
.TP 15
.B
pset
Control port for the task's processor set.
.TP 15
.B
policy
New policy.
.TP 15
.B
param[param_count]
Scheduling parameters for the new policy.
.TP 15
.B
change_threads
Change policy of existing threads if TRUE.

.SH DESCRIPTION
The policy of a task is used only for creation of new threads; a new thread's
scheduling policy is set to the enclosing task's policy.
.B task_set_default_policy
changes this task policy.  It also sets the policies and scheduling
parameters of all threads
in the task to this new policy if 
.B change_threads
is TRUE.  Existing threads are not affected otherwise.  If this policy
is not available for a thread in the task, or the
change violates the policy limit values of some threads, as many threads as
possible will be changed and an error code will be returned.

.SH DIAGNOSTICS
.TP 25
KERN_SUCCESS
The call succeeded.
.TP 25
KERN_INVALID_ARGUMENT
.B task
is not a task, or
.B policy
is not a valid policy, or
.B param
is not valid for the policy.
.TP 25
KERN_FAILURE
.B change_threads
was TRUE and the attempt to change the policy of at least one existing
thread failed because the thread's processor set does not run that
policy, or
the new policy parameters would have exceeded that thread's
policy limit values.

.SH BUGS
Availability limited.

.SH SEE ALSO
.B thread_set_policy_param, processor_set_policy_limit.



