.\" 
.\" Mach Operating System
.\" Copyright (c) 1993 Carnegie Mellon University
.\" All Rights Reserved.
.\" 
.\" Permission to use, copy, modify and distribute this software and its
.\" documentation is hereby granted, provided that both the copyright
.\" notice and this permission notice appear in all copies of the
.\" software, derivative works or modified versions, and any portions
.\" thereof, and that both notices appear in supporting documentation.
.\" 
.\" CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
.\" CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
.\" ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
.\" 
.\" Carnegie Mellon requests users of this software to return to
.\" 
.\"  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
.\"  School of Computer Science
.\"  Carnegie Mellon University
.\"  Pittsburgh PA 15213-3890
.\" 
.\" any improvements or extensions that they make and grant Carnegie Mellon 
.\" the rights to redistribute these changes.
.\"
.\"
.\" HISTORY
.\" $Log:	thread_set_periodic_timers.man,v $
.\" Revision 2.2  93/12/07  13:58:16  dbg
.\" 	Created.
.\" 
.\"
.TH thread_set_periodic_timers 2 12/7/93
.CM 4
.SH NAME
.nf
thread_set_periodic_timers  \-  sets period and deadline timers for thread
thread_get_periodic_timers  \-  returns period and deadline timers for thread
.SH SYNOPSIS
.nf
.ft B
#include <mach.h>

.nf
.ft B
kern_return_t thread_set_periodic_timers(
	thread_t		thread,
	mach_timer_t		periodic_timer,
	mach_timer_t		deadline_timer)


.fi
.ft P
.nf
.ft B
kern_return_t thread_get_periodic_timers(
	thread_t		thread,
	mach_timer_t		*periodic_timer_p,	/* OUT */
	mach_timer_t		*deadline_timer_p)	/* OUT */


.fi
.ft P
.SH ARGUMENTS
.TP 15
.B
thread
The thread for which to set the timers
.TP 15
.B
periodic_timer
The periodic timer for the thread.
.TP 15
.B
deadline timer
The deadline timer for the thread.

.SH DESCRIPTION

A periodic thread owns two timers: the
.B periodic,
or wakeup timer, which awakens the thread at the start of each period
of execution, and the
.B deadline
timer, which suspends the thread and sends a message to a deadline handler
if the thread overruns the execution deadline for the current period.

.B thread_set_periodic_timers
installs the period and deadline timers for the thread;
.B thread_get_periodic_timers
returns them.  Either timer may be
.B MACH_TIMER_NULL,
if the thread is not a periodic thread or if the timers have not been
installed.  A thread without a periodic timer will not be awakened
automatically.  However, timer_arm (on another timer) or the timeout
parameter to mach_msg may be used to awaken the thread.  A thread without
a deadline timer will proceed with its computation even if the computation
runs into the next period of execution.

.SH DIAGNOSTICS
.TP 25
KERN_SUCCESS
The timers were set or returned.
.TP 25
KERN_INVALID_ARGUMENT
.B thread
is not a thread, or
.B periodic_timer
or
.B deadline_timer
is neither a timer nor MACH_TIMER_NULL.

.SH SEE ALSO
.B periodic_thread_create, periodic_thread_restart, timer_arm

.SH BUGS
Availability limited.
