.\" 
.\" Mach Operating System
.\" Copyright (c) 1993 Carnegie Mellon University
.\" All Rights Reserved.
.\" 
.\" Permission to use, copy, modify and distribute this software and its
.\" documentation is hereby granted, provided that both the copyright
.\" notice and this permission notice appear in all copies of the
.\" software, derivative works or modified versions, and any portions
.\" thereof, and that both notices appear in supporting documentation.
.\" 
.\" CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
.\" CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
.\" ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
.\" 
.\" Carnegie Mellon requests users of this software to return to
.\" 
.\"  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
.\"  School of Computer Science
.\"  Carnegie Mellon University
.\"  Pittsburgh PA 15213-3890
.\" 
.\" any improvements or extensions that they make and grant Carnegie Mellon
.\" the rights to redistribute these changes.
.\" 
.\" 
.\" HISTORY
.\" $Log:	thread_set_policy.man,v $
.\" Revision 2.2  93/12/07  13:59:11  dbg
.\" 	Created.
.\" 
.\" 
.TH thread_set_policy 2 9/22/93
.CM 4
.SH NAME
.nf
thread_set_policy  \-  set scheduling policy for a thread
.SH SYNOPSIS
.nf
.ft B
#include <mach.h>

.nf
.ft B
kern_return_t thread_set_policy(
	thread_t	thread,
	processor_set_t	pset,
	int		policy,
	policy_param_t	param,
	natural_t	param_count);


.fi
.ft P
.SH ARGUMENTS
.TP 15
.B
thread
Thread to set policy for.
.TP 15
.B
pset
The control port for the processor set to which thread is assigned.
.TP 15
.B
policy
Policy to set.
.TP 15
.B
param[param_count]
Policy-specific parameters.

.SH DESCRIPTION
.B thread_set_policy
changes the scheduling policy for 
.B thread
to 
.B policy.

.B param
is policy-dependent scheduling information.  The base system supports
three scheduling policies: 
.B POLICY_FIXEDPRI,
.B POLICY_TIMESHARE
and 
.B POLICY_BACKGROUND,
defined in <mach/policy.h>; this file is included by mach.h.  Fixed-priority
threads are chosen to run before any timesharing threads; in turn,
timesharing threads are chosen before any background threads.  Within
each policy, the policy parameters for each thread determines which
thread is chosen to run.

.B POLICY_BACKGROUND

Background threads run only if no other threads in the system are runnable.
They are scheduled in a round-robin fashion.
.B param
is not used.

.B POLICY_TIMESHARE

Timesharing threads have a base priority value ranging from 0 (highest)
to 31 (lowest).  The actual priority used for scheduling threads is
the sum of the base priority value and an increment (toward lower priority)
derived from the thread's run time.
.B param
is the base priority value for the thread.

.B POLICY_FIXEDPRI

Fixed-priority threads run in order of their priorities, which range
from 0 (highest) to 31 (lowest).  A thread may also be designated
as round-robin or first-in-first-out.  The processor multiplexes between
round-robin threads of equal priority at each quantum expiration.  In
contrast, a first-in-first-out thread will run until it blocks, if no
higher-priority threads are runnable.

.B param
has two components:  the thread's priority, and whether the thread
is round-robin (TRUE) or first-in-first-out (FALSE).

Processor sets may restrict the allowed policies, so this call will fail
if the processor set to which 
.B thread
is currently assigned does not permit 
.B policy.
Moreover, the thread's processor set control port
must be presented to allow changing the thread's policy, since
a malfunctioning fixed-priority thread at priority 0 would prevent
all other threads in the system from running.

.SH DIAGNOSTICS
.TP 25
KERN_SUCCESS
The call succeeded.
.TP 25
KERN_INVALID_ARGUMENT
.B thread
is not a thread, or
.B policy
is not a recognized policy, or
.B pset
is not the thread's current processor set.
.TP 25
KERN_FAILURE
The processor set to which 
.B thread
is currently assigned does not permit 
.B policy.

.SH BUGS
Availability limited.

.SH SEE ALSO
.B task_set_default_policy, processor_set_policy_add, host_info



