 .\" 
.\" Mach Operating System
.\" Copyright (c) 1993 Carnegie Mellon University
.\" All Rights Reserved.
.\" 
.\" Permission to use, copy, modify and distribute this software and its
.\" documentation is hereby granted, provided that both the copyright
.\" notice and this permission notice appear in all copies of the
.\" software, derivative works or modified versions, and any portions
.\" thereof, and that both notices appear in supporting documentation.
.\" 
.\" CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
.\" CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
.\" ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
.\" 
.\" Carnegie Mellon requests users of this software to return to
.\" 
.\"  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
.\"  School of Computer Science
.\"  Carnegie Mellon University
.\"  Pittsburgh PA 15213-3890
.\" 
.\" any improvements or extensions that they make and grant Carnegie Mellon
.\" the rights to redistribute these changes.
.\" 
.\" 
.\" HISTORY
.\" $Log:	timer_sleep.man,v $
.\" Revision 2.2  93/12/07  14:01:35  dbg
.\" 	Created.
.\" 
.\" 
.TH timer_sleep 2 9/22/93
.CM 4
.SH NAME
.nf
timer_sleep  \-  sleep on a timer until a specified time
.SH SYNOPSIS
.nf
.ft B
#include <mach.h>

.nf
.ft B
kern_return_t
timer_sleep(
	mach_timer_t	timer,
	time_spec_t	expire_time,
	int		flags,
	time_spec_t	*wakeup_time);	/* out */


.fi
.ft P
.SH ARGUMENTS
.TP 15
.B
timer
The timer on which the calling thread will sleep.
.TP 15
.B expire_time
The time at which the timer will expire, measured
against
.B timer's
clock.
.B flags
Options for the expiration time.
.TP 15
.B
wakeup_time
Returns the absolute time at which the thread was
awakened.


.SH DESCRIPTION
.B timer_sleep
makes the current thread sleep on a timer until
a given time.  The expiration time is absolute, if
.B TIMER_ABSOLUTE
is set in
.B flags;
otherwise, the expiration time is relative to
the current time (of
.B timer's
clock).

When the timer expires, the thread is awakened,
and the absolute time at which it was awakened
is returned through
.B wakeup_time.
This will be the same as
.B expire_time,
unless the expiration time is relative or
the thread was prematurely awakened.

The thread may be awakened before the expiration
time is up by calling
.B thread_abort
on the thread, or by calling
.B timer_cancel
on the timer.  The thread will also be awakened if
the timer is destroyed (by a call to
.B timer_terminate
or if all send rights to the timer's port have been
deallocated).


.SH DIAGNOSTICS
.TP 25
KERN_SUCCESS
Operation completed successfully
.TP 25
KERN_INVALID_ARGUMENT
.B timer
is not a timer, or
.B expire_time
is not a valid time specification.
.TP 25
KERN_FAILURE
.B timer
is currently armed.

.SH BUGS

.SH SEE ALSO
.B timer_create, timer_terminate, timer_arm, timer_cancel, thread_abort.

